searchNodes=[{"ref":"EtsDeque.html","title":"EtsDeque","type":"module","doc":"EtsDeque is an Elixir implementation of a double-ended queue (deque), using Erlang&#39;s ETS library as a backing store. Using ETS ensures that all functions in the EtsDeque module execute in amortized O(1) time with a minimum of memory allocations, offering bounded or unbounded operation with high performance and favorable RAM usage. Using ETS also means that EtsDeque is not a purely functional data structure, and is not suitable for direct concurrent usage in multiple processes. Use the EtsDeque.Server GenServer if you would like safe access to an EtsDeque from multiple processes. You can push items onto, pop items from, or peek at items from the head or tail of the queue. Additionally, any item can be accessed or replaced by its index using at/2 and replace_at/3. EtsQueue implements Elixir&#39;s Access behaviour and Enumerable and Collectable protocols, so code like deque[0] and Enum.count(deque) and Enum.into([1, 2, 3], EtsDeque.new()) works as it should. Example iex&gt; deque = EtsDeque.new(3) iex&gt; {:ok, deque} = EtsDeque.push_head(deque, :moe) iex&gt; {:ok, deque} = EtsDeque.push_tail(deque, :larry) iex&gt; {:ok, deque} = EtsDeque.push_tail(deque, :curly) iex&gt; :error = EtsDeque.push_head(deque, :shemp) ## deque is full iex&gt; {:ok, :curly, deque} = EtsDeque.pop_tail(deque) iex&gt; {:ok, deque} = EtsDeque.push_tail(deque, :shemp) iex&gt; Enum.to_list(deque) [:moe, :larry, :shemp]"},{"ref":"EtsDeque.html#at/2","title":"EtsDeque.at/2","type":"function","doc":"Returns the item at the given index, where index 0 is the head. Returns :error if index is out of bounds."},{"ref":"EtsDeque.html#at!/2","title":"EtsDeque.at!/2","type":"function","doc":"Returns the item at the given index, where index 0 is the head. Raises ArgumentError if index is out of bounds."},{"ref":"EtsDeque.html#length/1","title":"EtsDeque.length/1","type":"function","doc":"Returns the number of items in the given deque. Equivalent to deque.length."},{"ref":"EtsDeque.html#new/1","title":"EtsDeque.new/1","type":"function","doc":"Creates a deque, optionally limited to a given size."},{"ref":"EtsDeque.html#peek_head/1","title":"EtsDeque.peek_head/1","type":"function","doc":"Returns the item at the head of the queue, or :error if the queue is empty."},{"ref":"EtsDeque.html#peek_head!/1","title":"EtsDeque.peek_head!/1","type":"function","doc":"Returns the item at the head of the queue, or raises ArgumentError if the queue is empty."},{"ref":"EtsDeque.html#peek_tail/1","title":"EtsDeque.peek_tail/1","type":"function","doc":"Returns the item at the tail of the queue, or :error if the queue is empty."},{"ref":"EtsDeque.html#peek_tail!/1","title":"EtsDeque.peek_tail!/1","type":"function","doc":"Returns the item at the tail of the queue, or raises ArgumentError if the queue is empty."},{"ref":"EtsDeque.html#pop_head/1","title":"EtsDeque.pop_head/1","type":"function","doc":"Removes the item at the head of the queue, returning it along with the updated deque. Returns :error if queue is empty."},{"ref":"EtsDeque.html#pop_head!/1","title":"EtsDeque.pop_head!/1","type":"function","doc":"Removes the item at the head of the queue, returning it along with the updated deque. Raises ArgumentError if queue is empty."},{"ref":"EtsDeque.html#pop_tail/1","title":"EtsDeque.pop_tail/1","type":"function","doc":"Removes the item at the tail of the queue, returning it along with the updated deque. Returns :error if queue is empty."},{"ref":"EtsDeque.html#pop_tail!/1","title":"EtsDeque.pop_tail!/1","type":"function","doc":"Removes the item at the tail of the queue, returning it along with the updated deque. Raises ArgumentError if queue is empty."},{"ref":"EtsDeque.html#push_head/2","title":"EtsDeque.push_head/2","type":"function","doc":"Adds an item onto the head of the queue. Returns the updated deque, or :error if the queue is full."},{"ref":"EtsDeque.html#push_head!/2","title":"EtsDeque.push_head!/2","type":"function","doc":"Adds an item onto the head of the queue. Returns the updated deque, or raises ArgumentError if the queue is full."},{"ref":"EtsDeque.html#push_tail/2","title":"EtsDeque.push_tail/2","type":"function","doc":"Adds an item onto the tail of the queue. Returns the updated deque, or :error if the queue is full."},{"ref":"EtsDeque.html#push_tail!/2","title":"EtsDeque.push_tail!/2","type":"function","doc":"Adds an item onto the tail of the queue. Returns the updated deque, or raises ArgumentError if the queue is full."},{"ref":"EtsDeque.html#replace_at/3","title":"EtsDeque.replace_at/3","type":"function","doc":"Replaces the item at the given index, returning the updated deque. Returns :error if index is out of bounds."},{"ref":"EtsDeque.html#replace_at!/3","title":"EtsDeque.replace_at!/3","type":"function","doc":"Replaces the item at the given index, returning the updated deque. Raises ArgumentError if index is out of bounds."},{"ref":"EtsDeque.html#size/1","title":"EtsDeque.size/1","type":"function","doc":"Returns the maximum capacity of the given deque. Equivalent to deque.size."},{"ref":"EtsDeque.html#t:t/0","title":"EtsDeque.t/0","type":"type","doc":""},{"ref":"EtsDeque.Server.html","title":"EtsDeque.Server","type":"module","doc":"EtsDeque.Server is a GenServer wrapper around an EtsDeque. It provides safe access to a deque from multiple processes, ensuring each operation on the deque is atomic. Example iex&gt; {:ok, pid} = EtsDeque.Server.start_link(size: 3) iex&gt; :ok = EtsDeque.Server.push_head(pid, :moe) iex&gt; :ok = EtsDeque.Server.push_tail(pid, :larry) iex&gt; :ok = EtsDeque.Server.push_tail(pid, :curly) iex&gt; :error = EtsDeque.Server.push_tail(pid, :shemp) ## deque is full iex&gt; {:ok, :curly} = EtsDeque.Server.pop_tail(pid) iex&gt; :ok = EtsDeque.Server.push_tail(pid, :shemp) iex&gt; EtsDeque.Server.execute(pid, fn deque -&gt; Enum.to_list(deque) end) [:moe, :larry, :shemp]"},{"ref":"EtsDeque.Server.html#at/3","title":"EtsDeque.Server.at/3","type":"function","doc":"Returns the item at the given index, where index 0 is the head. Returns :error if index is out of bounds."},{"ref":"EtsDeque.Server.html#child_spec/1","title":"EtsDeque.Server.child_spec/1","type":"function","doc":"Returns a specification to start this module under a supervisor. See Supervisor."},{"ref":"EtsDeque.Server.html#deque/2","title":"EtsDeque.Server.deque/2","type":"function","doc":"Returns the deque. Ensuring that no other process mutates the deque after it is returned is the caller&#39;s responsibility. See execute/3 for a safer alternative."},{"ref":"EtsDeque.Server.html#execute/3","title":"EtsDeque.Server.execute/3","type":"function","doc":"Executes fun.(deque), ensuring no other process is accessing the deque at the same time. Returns the result."},{"ref":"EtsDeque.Server.html#length/2","title":"EtsDeque.Server.length/2","type":"function","doc":"Returns the number of items in the given deque."},{"ref":"EtsDeque.Server.html#peek_head/2","title":"EtsDeque.Server.peek_head/2","type":"function","doc":"Returns the item at the head of the queue, or :error if the queue is empty."},{"ref":"EtsDeque.Server.html#peek_tail/2","title":"EtsDeque.Server.peek_tail/2","type":"function","doc":"Returns the item at the tail of the queue, or :error if the queue is empty."},{"ref":"EtsDeque.Server.html#pop_head/2","title":"EtsDeque.Server.pop_head/2","type":"function","doc":"Removes the item at the head of the queue, returning it along with the updated deque. Returns :error if queue is empty."},{"ref":"EtsDeque.Server.html#pop_tail/2","title":"EtsDeque.Server.pop_tail/2","type":"function","doc":"Removes the item at the tail of the queue, returning it along with the updated deque. Returns :error if queue is empty."},{"ref":"EtsDeque.Server.html#push_head/3","title":"EtsDeque.Server.push_head/3","type":"function","doc":"Adds an item onto the head of the queue. Returns the updated deque, or :error if the queue is full."},{"ref":"EtsDeque.Server.html#push_tail/3","title":"EtsDeque.Server.push_tail/3","type":"function","doc":"Adds an item onto the tail of the queue. Returns the updated deque, or :error if the queue is full."},{"ref":"EtsDeque.Server.html#replace_at/4","title":"EtsDeque.Server.replace_at/4","type":"function","doc":"Replaces the item at the given index, returning the updated deque. Returns :error if index is out of bounds."},{"ref":"EtsDeque.Server.html#size/2","title":"EtsDeque.Server.size/2","type":"function","doc":"Returns the maximum capacity of the given deque."}]